@@ -70,6 +70,18 @@
 #include <resolv.h>
 #include <netdb.h>
 
+#ifndef HAVE_SOCKADDR_STORAGE
+struct sockaddr_storage {
+#ifdef HAVE_SOCKADDR_LEN
+		unsigned char ss_len;
+		unsigned char ss_family;
+#else
+        unsigned short ss_family;
+#endif
+        char info[126];
+};
+#endif
+
 #ifndef socklen_t
 #  ifdef GNET_APPLE_DARWIN
 #    define socklen_t int	/* socklen_t is int in Darwin */
@@ -80,8 +92,9 @@
 
 #define GNET_CLOSE_SOCKET(SOCKFD) close(SOCKFD)
 
-/* Use gnet_private_iochannel_new() to create iochannels */
-#define GNET_SOCKET_IOCHANNEL_NEW(SOCKFD) g_io_channel_unix_new(SOCKFD)
+/* Use gnet_private_io_channel_new() to create iochannels */
+#define GNET_SOCKET_IO_CHANNEL_NEW(SOCKFD) g_io_channel_unix_new(SOCKFD)
+
 
 #else	/*********** Windows specific ***********/
 
@@ -91,21 +104,91 @@
 #include <ws2tcpip.h>
 
 #define socklen_t gint32
+#define in_addr_t guint32
 
 #define GNET_CLOSE_SOCKET(SOCKFD) closesocket(SOCKFD)
 
-/* Use gnet_private_iochannel_new() to create iochannels */
-#define GNET_SOCKET_IOCHANNEL_NEW(SOCKFD) g_io_channel_win32_new_socket(SOCKFD)
+/* Use gnet_private_io_channel_new() to create iochannels */
+#define GNET_SOCKET_IO_CHANNEL_NEW(SOCKFD) g_io_channel_win32_new_socket(SOCKFD)
 
 #endif	/*********** End Windows specific ***********/
 
 #ifndef INET_ADDRSTRLEN
 #define INET_ADDRSTRLEN 16
+#endif
+
+#ifndef INET6_ADDRSTRLEN
 #define INET6_ADDRSTRLEN 46
 #endif
 
-#define GNET_SOCKADDR_IN(s) (*((struct sockaddr_in*) &s))
-#define GNET_ANY_IO_CONDITION  (G_IO_IN|G_IO_OUT|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL)
+#ifndef IN_LOOPBACKNET /* not defined in Win32 */
+#define IN_LOOPBACKNET 127
+#endif
+
+
+#define GNET_SOCKADDR_IN(s)    	(*((struct sockaddr_in*) &s))
+#define GNET_SOCKADDR_SA(s)	(*((struct sockaddr*) &s))
+#define GNET_SOCKADDR_SA4(s)	(*((struct sockaddr_in*) &s))
+#define GNET_SOCKADDR_FAMILY(s) ((s).ss_family)
+
+#ifdef HAVE_IPV6
+#define GNET_SOCKADDR_SA6(s)	(*((struct sockaddr_in6*) &s))
+#define GNET_SOCKADDR_ADDRP(s)	(((s).ss_family == AF_INET)?\
+                                  (void*)&((struct sockaddr_in*)&s)->sin_addr:\
+                                  (void*)&((struct sockaddr_in6*)&s)->sin6_addr)
+#define GNET_SOCKADDR_ADDR32(s,n)(((s).ss_family == AF_INET)?\
+                                  ((struct sockaddr_in*)&s)->sin_addr.s_addr:\
+                                  *(guint32*)&((struct sockaddr_in6*)&s)->sin6_addr.s6_addr[(n)*4])
+#define GNET_SOCKADDR_ADDR32_SET(s,n,a) if ((s).ss_family == AF_INET) \
+                                          ((struct sockaddr_in*)&s)->sin_addr.s_addr = a; \
+                                        else \
+                                          *(guint32*)&((struct sockaddr_in6*)&s)->sin6_addr.s6_addr[(n)*4] = a;
+#define GNET_SOCKADDR_ADDRLEN(s) (((s).ss_family == AF_INET)?\
+				 sizeof(struct in_addr):\
+				 sizeof(struct in6_addr))
+#define GNET_SOCKADDR_PORT(s)	(((s).ss_family == AF_INET)?\
+                                  ((struct sockaddr_in*)&s)->sin_port:\
+                                  ((struct sockaddr_in6*)&s)->sin6_port)
+#define GNET_SOCKADDR_PORT_SET(s, p)	if ((s).ss_family == AF_INET)\
+                                          ((struct sockaddr_in*)&(s))->sin_port = p;\
+                                        else \
+                                          ((struct sockaddr_in6*)&(s))->sin6_port = p;
+#define GNET_SOCKADDR_LEN(s)	(((s).ss_family == AF_INET)?\
+                                  sizeof(struct sockaddr_in):\
+                                  sizeof(struct sockaddr_in6))
+#else /* NO IPV6 */
+
+#define GNET_SOCKADDR_ADDRP(s)	(void*)&((struct sockaddr_in*)&s)->sin_addr
+#define GNET_SOCKADDR_ADDR32(s,n)((struct sockaddr_in*)&s)->sin_addr.s_addr
+#define GNET_SOCKADDR_ADDRLEN(s) sizeof(struct in_addr)
+#define GNET_SOCKADDR_PORT(s)	((struct sockaddr_in*)&s)->sin_port
+#define GNET_SOCKADDR_LEN(s)	sizeof(struct sockaddr_in)
+#define GNET_SOCKADDR_PORT_SET(s, p) ((struct sockaddr_in*)&(s))->sin_port = p;
+
+#endif
+
+
+#ifdef HAVE_SOCKADDR_SA_LEN
+#define GNET_SOCKADDR_SET_SS_LEN(s) do{(s).ss_len = GNET_SOCKADDR_LEN(s);}while(0)
+#else
+#define GNET_SOCKADDR_SET_SS_LEN(s) while(0){} /* do nothing */
+#endif
+
+
+#define GNET_INETADDR_SA(i)         GNET_SOCKADDR_SA((i)->sa)
+#define GNET_INETADDR_SA4(i)        GNET_SOCKADDR_SA4((i)->sa)
+#define GNET_INETADDR_SA6(i)        GNET_SOCKADDR_SA6((i)->sa) 
+#define GNET_INETADDR_FAMILY(i)     GNET_SOCKADDR_FAMILY((i)->sa)
+#define GNET_INETADDR_ADDRP(i)      GNET_SOCKADDR_ADDRP((i)->sa)
+#define GNET_INETADDR_ADDR32(i,n)   GNET_SOCKADDR_ADDR32((i)->sa,(n))
+#define GNET_INETADDR_ADDRLEN(i)    GNET_SOCKADDR_ADDRLEN((i)->sa)
+#define GNET_INETADDR_PORT(i)       GNET_SOCKADDR_PORT((i)->sa)
+#define GNET_INETADDR_PORT_SET(i, p)	GNET_SOCKADDR_PORT_SET((i)->sa, p)
+#define GNET_INETADDR_LEN(i)        GNET_SOCKADDR_LEN((i)->sa)
+#define GNET_INETADDR_SET_SS_LEN(i) GNET_SOCKADDR_SET_SS_LEN((i)->sa)
+
+
+#define GNET_ANY_IO_CONDITION   (G_IO_IN|G_IO_OUT|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL)
 
 
 #ifdef __cplusplus
@@ -123,77 +206,91 @@ extern "C" {
 */
 
 
+
 struct _GUdpSocket
 {
-  gint sockfd;			/* private */
-  struct sockaddr sa;		/* private (Why not an InetAddr?) */
+  gint sockfd;
   guint ref_count;
   GIOChannel* iochannel;
+  struct sockaddr_storage sa;
+};
+
+struct _GMcastSocket
+{
+  gint sockfd;
+  guint ref_count;
+  GIOChannel* iochannel;
+  struct sockaddr_storage sa;
 };
 
 struct _GTcpSocket
 {
   gint sockfd;
-  struct sockaddr sa;		/* Why not an InetAddr? */
   guint ref_count;
   GIOChannel* iochannel;
+  struct sockaddr_storage sa;
+  /* sa is remote host for clients, local host for servers */
 
   GTcpSocketAcceptFunc accept_func;
   gpointer accept_data;
   guint	accept_watch;
 };
 
-struct _GUnixSocket
-{
-  gint sockfd;
-  struct sockaddr sa;
-  guint ref_count;
-  gboolean server;
-  GIOChannel *iochannel;
-};
-
-struct _GMcastSocket
-{
-  gint sockfd;
-  struct sockaddr sa;
-  guint ref_count;
-  GIOChannel* iochannel;
-};
-
 struct _GInetAddr
 {
   gchar* name;
-  struct sockaddr sa;
   guint ref_count;
+  struct sockaddr_storage sa;
 };
 
 
+#ifndef GNET_WIN32  /*********** Unix specific ***********/
+
+struct _GUnixSocket
+{
+  gint sockfd;
+  guint ref_count;
+  GIOChannel *iochannel;
+  struct sockaddr_un sa;
+
+  gboolean server;
+};
+
+#endif	/*********** End Unix specific ***********/
+
+
+
 /* **************************************** */
 /* Async functions			*/
 
-gboolean gnet_inetaddr_new_async_cb (GIOChannel* iochannel, 
-				     GIOCondition condition, 
-				     gpointer data);
+gboolean gnet_inetaddr_new_list_async_cb (GIOChannel* iochannel, 
+					  GIOCondition condition, 
+					  gpointer data);
 
-typedef struct _GInetAddrAsyncState 
+
+typedef struct _GInetAddrNewListState 
 {
-  GInetAddr* 	ia;
-  GInetAddrNewAsyncFunc func;
+  GList*	ias;
+  gint		port;
+  GInetAddrNewListAsyncFunc func;
   gpointer 	data;
+
 #ifndef GNET_WIN32		/* UNIX */
+  gboolean 	in_callback;
 #ifdef HAVE_LIBPTHREAD		/* UNIX pthread	*/
   pthread_mutex_t mutex;
   gboolean	is_cancelled;
   gboolean	lookup_failed;
   guint 	source;
+
 #else			       	/* UNIX process	*/
   int 		fd;
   pid_t 	pid;
   GIOChannel* 	iochannel;
   guint 	watch;
-  guchar 	buffer[16];
   int 		len;
-  gboolean 	in_callback;
+  guchar 	buffer[256];
+
 #endif
 #else				/* Windows */
   int 		WSAhandle;
@@ -202,7 +299,18 @@ typedef struct _GInetAddrAsyncState
   gboolean 	in_callback;
 #endif
 
-} GInetAddrAsyncState;
+} GInetAddrNewListState;
+
+
+
+typedef struct _GInetAddrNewState 
+{
+  GInetAddrNewListAsyncID	list_id;
+  GInetAddrNewAsyncFunc 	func;
+  gpointer 			data;
+  gboolean			in_callback;
+
+} GInetAddrNewState;
 
 
 
@@ -245,16 +353,15 @@ gboolean gnet_tcp_socket_new_async_cb (GIOChannel* iochannel,
 
 typedef struct _GTcpSocketAsyncState 
 {
-  GTcpSocket* socket;
+  GTcpSocket* 		 socket;
   GTcpSocketNewAsyncFunc func;
-  gpointer data;
-  gint flags;
-  GIOChannel* iochannel;
-  guint connect_watch;
+  gpointer 		 data;
+  gint 			 flags;
+  GIOChannel* 		 iochannel;
+  guint 		 connect_watch;
 #ifdef GNET_WIN32
-  gint errorcode;
+  gint 			 errorcode;
 #endif
-
 } GTcpSocketAsyncState;
 
 #ifdef GNET_WIN32
@@ -274,23 +381,23 @@ typedef struct _SocketWatchAsyncState
 } SocketWatchAsyncState;
 #endif
 
-void gnet_tcp_socket_connect_inetaddr_cb(GInetAddr* inetaddr, 
-					 GInetAddrAsyncStatus status, 
-					 gpointer data);
+void gnet_tcp_socket_connect_inetaddr_cb (GList* ia_list, gpointer data);
 
-void gnet_tcp_socket_connect_tcp_cb(GTcpSocket* socket, 
-				    GTcpSocketConnectAsyncStatus status, 
-				    gpointer data);
+void gnet_tcp_socket_connect_tcp_cb(GTcpSocket* socket, gpointer data);
 
 typedef struct _GTcpSocketConnectState 
 {
-  GInetAddr* ia;
-  GTcpSocketConnectAsyncFunc func;
-  gpointer data;
+  GList* ia_list;
+  GList* ia_next;
 
   gpointer inetaddr_id;
   gpointer tcp_id;
 
+  gboolean in_callback;
+
+  GTcpSocketConnectAsyncFunc func;
+  gpointer data;
+
 } GTcpSocketConnectState;
 
 
@@ -319,13 +426,9 @@ extern HANDLE gnet_hostent_Mutex;
 
 /* Private/Experimental functions */
 
+GIOChannel* gnet_private_io_channel_new (int sockfd);
 
-GIOChannel* gnet_private_iochannel_new (int sockfd);
-
-GInetAddr* gnet_private_inetaddr_sockaddr_new(const struct sockaddr sa);
-
-struct sockaddr gnet_private_inetaddr_get_sockaddr(const GInetAddr* ia);
-/* gtk-doc doesn't like this function...  (but it will be fixed.) */
+int gnet_private_create_listen_socket (int type, const GInetAddr* iface, int port, struct sockaddr_storage* sa);
 
 
 #ifdef __cplusplus
