@@ -31,11 +31,13 @@
 #include "tcp.h"
 #include "socks.h"
 #include "pack.h"
-#include "url.h"
+#include "uri.h"
 #include "conn.h"
 #include "server.h"
 #include "md5.h"
 #include "sha.h"
+#include "ipv6.h"
+#include "base64.h"
 
 #ifndef GNET_WIN32
 #  include "unix.h"
@@ -45,6 +47,12 @@
 extern "C" {
 #endif /* __cplusplus */
 
+/**
+ * GNET_EXPORT:
+ *
+ * Declares a variable exported.  Used interally.
+ *
+ **/
 #ifdef GNET_WIN32
 #  define GNET_EXPORT extern __declspec(dllimport)
 #else
@@ -57,15 +65,18 @@ GNET_EXPORT const guint gnet_micro_version;
 GNET_EXPORT const guint gnet_interface_age;
 GNET_EXPORT const guint gnet_binary_age;
 
+
+
 /**
- *  GNET_CHECK_VERSION:
+ *  GNET_CHECK_VERSION
  *  @major: Major version number
  *  @minor: Minor version number
  *  @micro: Micro version number
  *
- *  Check if the version given is compatable with this version of the
+ *  Checks if the version given is compatable with this version of the
  *  library.  For example, GNET_CHECK_VERSION(1.2.3) would return TRUE
- *  if the version is 1.2.5, and FALSE if 1.1.0.
+ *  if the version is 1.2.5, and FALSE if 1.1.0.  This can be used in
+ *  build tests.
  *
  **/
 #define GNET_CHECK_VERSION(major,minor,micro)    \
@@ -74,8 +85,14 @@ GNET_EXPORT const guint gnet_binary_age;
      (GNET_MAJOR_VERSION == (major) && GNET_MINOR_VERSION == (minor) && \
       GNET_MICRO_VERSION >= (micro)))
 
+
+
+
+
 void gnet_init (void);
 
+
+
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
