@@ -21,8 +21,8 @@
 #include <string.h>
 
 
-static guint strlenn(char* str, guint n);
-static void flipmemcpy(char* dst, char* src, guint n);
+static gsize strlenn(char* str, gsize n);
+static void flipmemcpy(char* dst, char* src, gsize n);
 
 
 #define MEMCPY(D,S,N)				\
@@ -71,10 +71,16 @@ static void flipmemcpy(char* dst, char* src, guint n);
   } while(0)
 
 
-static guint
-strlenn(char* str, guint n)
+/*
+
+  Get the length of string @str.  @str is at most @n characters long.
+  @str need not be NUL-terminated (which is why we pass @n).
+
+*/
+static gsize
+strlenn(char* str, gsize n)
 {
-  guint len = 0;
+  gsize len = 0;
 
   while (*str++ && len < n) ++len;
 
@@ -83,7 +89,7 @@ strlenn(char* str, guint n)
 
 
 static void 
-flipmemcpy(char* dst, char* src, guint n)
+flipmemcpy(char* dst, char* src, gsize n)
 {
   int nn = n;
 
@@ -123,7 +129,7 @@ flipmemcpy(char* dst, char* src, guint n)
     for (mult=(mult?mult:1); mult; --mult)			\
       {								\
         TYPE t;							\
-        g_return_val_if_fail (n + sizeof(TYPE) <= len, -1);	\
+        g_return_val_if_fail (n + sizeof(TYPE) <= length, -1);	\
         t = (TYPE) va_arg (args, VTYPE);                        \
         MEMCPY(buffer, (char*) &t, sizeof(TYPE));               \
         buffer += sizeof(TYPE);					\
@@ -141,7 +147,7 @@ flipmemcpy(char* dst, char* src, guint n)
         if (sizemode == 0)						\
           {								\
              TYPENATIVE t;						\
-             g_return_val_if_fail (n + sizeof(TYPENATIVE) <= len, -1);	\
+             g_return_val_if_fail (n + sizeof(TYPENATIVE) <= length, -1);\
              t = (TYPENATIVE) va_arg (args, VTYPE);                    	\
              MEMCPY(buffer, (char*) &t, sizeof(TYPENATIVE));            \
              buffer += sizeof(TYPENATIVE);				\
@@ -150,7 +156,7 @@ flipmemcpy(char* dst, char* src, guint n)
         else if (sizemode == 1)						\
           {								\
              TYPESTD t;							\
-             g_return_val_if_fail (n + sizeof(TYPESTD) <= len, -1);	\
+             g_return_val_if_fail (n + sizeof(TYPESTD) <= length, -1);	\
              t = (TYPESTD) va_arg (args, VTYPE);               		\
              LEMEMCPY(buffer, (char*) &t, sizeof(TYPESTD));             \
              buffer += sizeof(TYPESTD);					\
@@ -159,7 +165,7 @@ flipmemcpy(char* dst, char* src, guint n)
         else if (sizemode == 2)						\
           {								\
              TYPESTD t;							\
-             g_return_val_if_fail (n + sizeof(TYPESTD) <= len, -1);	\
+             g_return_val_if_fail (n + sizeof(TYPESTD) <= length, -1);	\
              t = (TYPESTD) va_arg (args, VTYPE);                       	\
              BEMEMCPY(buffer, (char*) &t, sizeof(TYPESTD));             \
              buffer += sizeof(TYPESTD);					\
@@ -174,47 +180,70 @@ flipmemcpy(char* dst, char* src, guint n)
 
 
 /**
- *  gnet_pack:
- *  @format: Pack format
- *  @buffer: Buffer to pack to
- *  @len: Length of buffer
- *  @Varargs: Variables to pack from
- *
- *  The pack format string is a list of types.  Each type is
- *  represented by a character.  Most types can be prefixed by an
- *  integer, which represents how many times it is repeated (eg,
- *  "4i2b" is equivalent to "iiiibb".
- *
- *  Native size/order is the default.  If the first character of
- *  FORMAT is < then little endian order and standard size are used.
- *  If the first character is > or !, then big endian (or network)
- *  order and standard size are used.  Standard sizes are 1 byte for
- *  chars, 2 bytes for shorts, and 4 bytes for ints and longs.
- *  x is a pad byte.  The pad byte is the NULL character.
- *
- *  b/B are signed/unsigned chars
- *
- *  h/H are signed/unsigned shorts
- *
- *  i/I are signed/unsigned ints
- *
- *  l/L are signed/unsigned longs
- *
- *  f/D are floats/doubles (always native order/size)
+ *  gnet_pack
+ *  @format: pack data format
+ *  @buffer: buffer to pack to
+ *  @length: length of @buffer
+ *  @Varargs: variables to pack from
  *  
- *  v is a void pointer (always native size)
+ *  Writes @Varargs to @buffer.  @format is a string that describes
+ *  the @Varargs and how they are to be packed.  This string is a list
+ *  of characters, each describing the type of an argument in
+ *  @Varargs.
  *
- *  s is a zero-terminated string.  REPEAT is repeat.
+ *  An example:
+ *
+ *  <informalexample>
+ *  <programlisting>
+ *  char buf[5];
+ *  int myint = 42;
+ *  int mybyte = 23;
+ *  &space;
+ *  gnet_pack("!ib", buf, sizeof(buf), myint, mybyte);
+ *  &comstart; Now buf is { 42, 0, 0, 0, 23 }; &comend;
+ *  </programlisting>
+ *  </informalexample>
+ *
+ *  As a shortcut, most types can be prefixed by an integer to specify
+ *  how many times the type is repeated.  For example, "4i2b" is
+ *  equivalent to "iiiibb".  This repeat argument is refered below to
+ *  as REPEAT.
+ *
+ *  Native byte order and sizes are used by default.  If the first
+ *  character of @format is < then little endian order and standard
+ *  sizes are used.  If the first character is > or !, then big endian
+ *  (or network) order and standard size are used.  Standard sizes are
+ *  1 byte for chars, 2 bytes for shorts, and 4 bytes for ints and
+ *  longs.
+ *
+ *  The types:
+ *
+ *  x is a NULL character.  It can be used for padding.  It does not
+ *  correspond to an argument in @Varargs.
+ *
+ *  b/B is a signed/unsigned char.
+ *
+ *  h/H is a signed/unsigned short.
+ *
+ *  i/I is a signed/unsigned int.
+ *
+ *  l/L is a signed/unsigned long.
+ *
+ *  f/D is a float/double (always native order/size).
+ *  
+ *  v is a void pointer (always native size).
+ *
+ *  s is a zero-terminated string.
  *
  *  S is a zero-padded string of maximum length REPEAT.  We write
  *  up-to a NULL character or REPEAT characters, whichever comes
  *  first.  We then write NULL characters up to a total of REPEAT
- *  characters.  Special case: If REPEAT is not specified, we write
+ *  characters.  Special case: if REPEAT is not specified, we write
  *  the string as a non-NULL-terminated string (note that it can't be
  *  unpacked easily then).
  *
- *  r is a byte array of NEXT bytes.  NEXT is the next argument and is
- *  an integer.  REPEAT is repeat.  (r is from "raw")
+ *  r is a byte array of NEXT bytes.  NEXT is the next argument passed
+ *  to gnet_pack() and is a gint.
  *
  *  R is a byte array of REPEAT bytes.  REPEAT must be specified.
  *
@@ -226,21 +255,21 @@ flipmemcpy(char* dst, char* src, guint n)
  *  Mnemonics: (B)yte, s(H)ort, (I)nteger, (F)loat, (D)ouble, (V)oid
  *  pointer, (S)tring, (R)aw
  *
- *  pack was mostly inspired by Python's pack, with some awareness of
- *  Perl's pack.  We don't do Python 0-repeat-is-alignment.  Submit a
- *  patch if you really want it.
+ *  Pack was inspired by Python's and Perl's pack.  It is more like
+ *  Python's than Perl's.  Note that in GNet, a repeat of 0 does not
+ *  align the data (as in Python).
  *
- *  Returns: bytes packed; -1 if error.
+ *  Returns: number of bytes packed; -1 on error.
  *
  **/
 gint
-gnet_pack (const gchar* format, gchar* buffer, const guint len, ...)
+gnet_pack (const gchar* format, gchar* buffer, const gint length, ...)
 {
   va_list args;
   gint rv;
   
-  va_start (args, len);
-  rv = gnet_vpack (format, buffer, len, args);
+  va_start (args, length);
+  rv = gnet_vpack (format, buffer, length, args);
   va_end (args);
 
   return rv;
@@ -249,42 +278,43 @@ gnet_pack (const gchar* format, gchar* buffer, const guint len, ...)
 
 /**
  *  gnet_pack_strdup
- *  @format: Pack format (see gnet_pack)
- *  @buffer: Pointer to buffer to allocate and pack to
- *  @Varargs: Variables to pack from
+ *  @format: pack data format
+ *  @bufferp: pointer to a buffer (buffer is caller owned)
+ *  @Varargs: variables to pack from
  *
- *  Packs the arguments into an allocated buffer.  Caller is
- *  responsible for deallocating the buffer.
+ *  Writes @Varargs into a buffer pointed to by @bufferp.  The buffer
+ *  is allocated by the function and is caller owned.  See gnet_pack()
+ *  for more information.
  *
- *  Returns: bytes packed; -1 if error.
+ *  Returns: bytes packed; -1 on error.
  *
  **/
 gint
-gnet_pack_strdup (const gchar* format, gchar** buffer, ...)
+gnet_pack_strdup (const gchar* format, gchar** bufferp, ...)
 {
   va_list args;
   gint size;
   gint rv;
   
   g_return_val_if_fail (format, -1);
-  g_return_val_if_fail (buffer, -1);
+  g_return_val_if_fail (bufferp, -1);
 
   /* Get size */
-  va_start (args, buffer);
+  va_start (args, bufferp);
   size = gnet_vcalcsize (format, args);
   va_end (args);
   g_return_val_if_fail (size >= 0, -1);
   if (size == 0)
     {
-      *buffer = NULL;
+      *bufferp = NULL;
       return 0;
     }
 
-  *buffer = g_new (gchar, size);
+  *bufferp = g_new (gchar, size);
 
   /* Pack */
-  va_start (args, buffer);
-  rv = gnet_vpack (format, *buffer, size, args);
+  va_start (args, bufferp);
+  rv = gnet_vpack (format, *bufferp, size, args);
   va_end (args);
 
   return rv;
@@ -294,14 +324,14 @@ gnet_pack_strdup (const gchar* format, gchar** buffer, ...)
 /* **************************************** */
 
 /**
- *  gnet_calcsize:
- *  @format: Pack format
- *  @Varargs: Variables
+ *  gnet_calcsize
+ *  @format: pack data format
+ *  @Varargs: variables
  *
- *  Calculate the size of the buffer needed to pack the given format.
- *  All arguments should be passed.
+ *  Calculates the size of the buffer needed to pack @Varargs by the
+ *  given format.  See gnet_pack() for more information.
  *
- *  Returns: number of bytes required to pack; -1 if error.
+ *  Returns: number of bytes required to pack; -1 on error.
  *  
  **/
 gint
@@ -319,22 +349,22 @@ gnet_calcsize (const gchar* format, ...)
 
 
 /**
- *  gnet_vcalcsize:
- *  @format: Pack format
+ *  gnet_vcalcsize
+ *  @format: pack data format
  *  @args: var args
  *
- *  Var arg interface to gnet_calcsize().  Size gnet_calcsize() for
+ *  Var arg interface to gnet_calcsize().  See gnet_calcsize() for
  *  additional information.
  *
- *  Returns: number of bytes required to pack; -1 if error.
+ *  Returns: number of bytes required to pack; -1 on error.
  *
  **/
 gint
 gnet_vcalcsize (const gchar* format, va_list args)
 {
-  guint n = 0;
+  gint n = 0;
   gchar* p = (gchar*) format;
-  gint mult = 0;
+  guint mult = 0;
   gint sizemode = 0;	/* 1 = little, 2 = big */
 
   if (!format)
@@ -362,8 +392,8 @@ gnet_vcalcsize (const gchar* format, va_list args)
 	case 'i':  { SIZE(int, int, gint32); 			break;  }
 	case 'I':  { SIZE(unsigned int, unsigned int, guint32); break;  }
 
-	case 'l':  { SIZE(long, int, gint32); 			break;  }
-	case 'L':  { SIZE(unsigned long, unsigned int, guint32); break;  }
+	case 'l':  { SIZE(long, long, gint32); 			break;  }
+	case 'L':  { SIZE(unsigned long, unsigned long, guint32); break;  }
 
 	case 'f':  { SIZE(float, double, float);		break;  }
 	case 'd':  { SIZE(double, double, double);		break;  }
@@ -472,29 +502,29 @@ gnet_vcalcsize (const gchar* format, va_list args)
 
 
 /**
- *  gnet_vpack:
- *  @format: Pack format (see gnet_pack)
- *  @buffer: Buffer to pack to
- *  @len: Length of buffer
+ *  gnet_vpack
+ *  @format: pack data format
+ *  @buffer: buffer to pack to
+ *  @length: length of @buffer
  *  @args: var args
  *
- *  Var arg interface to gnet_pack().  See gnet_pack() for format
+ *  Var arg interface to gnet_pack().  See gnet_pack() for more
  *  information.
  *
- *  Returns: bytes packed; -1 if error.
+ *  Returns: bytes packed; -1 on error.
  *
  **/
 gint
-gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
+gnet_vpack (const gchar* format, gchar* buffer, const gint length, va_list args)
 {
-  guint n = 0;
+  gint n = 0;
   gchar* p = (gchar*) format;
   guint mult = 0;
   gint sizemode = 0;	/* 1 = little, 2 = big */
 
   g_return_val_if_fail (format, -1);
   g_return_val_if_fail (buffer, -1);
-  g_return_val_if_fail (len, -1);
+  g_return_val_if_fail (length, -1);
 
   switch (*p)
     {
@@ -512,7 +542,7 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 	  {	
 	    for (mult=(mult?mult:1); mult; --mult)
 	      {
-		g_return_val_if_fail (n + 1 <= len, -1);
+		g_return_val_if_fail (n + 1 <= length, -1);
 		*buffer++ = 0;	++n; 
 	      }
 
@@ -530,8 +560,8 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 	case 'i':  { PACK2(int, int, gint32); 			break;  }
 	case 'I':  { PACK2(unsigned int, unsigned int, guint32); break;  }
 
-	case 'l':  { PACK2(long, int, gint32); 			break;  }
-	case 'L':  { PACK2(unsigned long, unsigned int, guint32); break;  }
+	case 'l':  { PACK2(long, long, gint32);			break;  }
+	case 'L':  { PACK2(unsigned long, unsigned long, guint32); break;  }
 
 	case 'f':  { PACK(float, double);			break;  }
 	case 'd':  { PACK(double, double);			break;  }
@@ -543,13 +573,13 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 	    for (mult=(mult?mult:1); mult; --mult)
 	      {
 		gchar* s; 
-		guint slen;
+		gsize slen;
 
 		s = va_arg (args, gchar*);
 		g_return_val_if_fail (s, -1);
 
 		slen = strlen(s);
-		g_return_val_if_fail (n + slen + 1 <= len, -1);
+		g_return_val_if_fail (n + slen + 1 <= length, -1);
 
 		memcpy (buffer, s, slen + 1);	/* include the 0 */
 		buffer += slen + 1;
@@ -569,10 +599,10 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 
 	    if (!mult)
 	      {
-		guint slen;
+		gsize slen;
 
 		slen = strlen(s);
-		g_return_val_if_fail (n + slen <= len, -1);
+		g_return_val_if_fail (n + slen <= length, -1);
 
 		memcpy (buffer, s, slen);	/* don't include the 0 */
 		buffer += slen;
@@ -582,7 +612,7 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 	      {
 		guint i;
 
-		g_return_val_if_fail (n + mult <= len, -1);
+		g_return_val_if_fail (n + mult <= length, -1);
 
 		for (i = 0; i < mult && s[i]; ++i)
 		  *buffer++ = s[i];
@@ -607,7 +637,7 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 		ln = va_arg (args, guint);
 
 		g_return_val_if_fail (s, -1);
-		g_return_val_if_fail (n + ln <= len, -1);
+		g_return_val_if_fail (n + ln <= length, -1);
 
 		memcpy(buffer, s, ln);
 		buffer += ln;
@@ -626,7 +656,7 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 	    g_return_val_if_fail (s, -1);
 
 	    g_return_val_if_fail (mult, -1);
-	    g_return_val_if_fail (n + mult <= len, -1);
+	    g_return_val_if_fail (n + mult <= length, -1);
 
 	    memcpy (buffer, s, mult);
 	    buffer += mult;
@@ -640,14 +670,14 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 	    for (mult=(mult?mult:1); mult; --mult)
 	      {
 		gchar* s;
-		guint slen;
+		gsize slen;
 
 		s = va_arg (args, char*);
 		g_return_val_if_fail (s, -1);
 
 		slen = strlen(s);
 		g_return_val_if_fail (n < 256, -1);
-		g_return_val_if_fail (n + slen + 1 <= len, -1);
+		g_return_val_if_fail (n + slen + 1 <= length, -1);
 
 		*buffer++ = slen;
 		memcpy (buffer, s, slen);   
@@ -684,7 +714,7 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
     for (mult=(mult?mult:1); mult; --mult)			\
       {								\
         TYPE* t;						\
-        g_return_val_if_fail (n + sizeof(TYPE) <= len, FALSE);	\
+        g_return_val_if_fail (n + sizeof(TYPE) <= length, FALSE);	\
         t = va_arg (args, TYPE*);                               \
         MEMCPY((char*) t, buffer, sizeof(TYPE));                \
         buffer += sizeof(TYPE);					\
@@ -701,7 +731,7 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
         if (sizemode == 0)						\
           {								\
              TYPENATIVE* t;						\
-             g_return_val_if_fail (n + sizeof(TYPENATIVE) <= len, -1);	\
+             g_return_val_if_fail (n + sizeof(TYPENATIVE) <= length, -1);	\
              t = va_arg (args, TYPENATIVE*);                           	\
              MEMCPY((char*) t, buffer, sizeof(TYPENATIVE));             \
              buffer += sizeof(TYPENATIVE);				\
@@ -710,7 +740,7 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
         else if (sizemode == 1)						\
           {								\
              TYPESTD* t;						\
-             g_return_val_if_fail (n + sizeof(TYPESTD) <= len, -1);	\
+             g_return_val_if_fail (n + sizeof(TYPESTD) <= length, -1);	\
              t = va_arg (args, TYPESTD*);                           	\
              LEMEMCPY((char*) t, buffer, sizeof(TYPESTD));              \
              buffer += sizeof(TYPESTD);					\
@@ -719,7 +749,7 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
         else if (sizemode == 2)						\
           {								\
              TYPESTD* t;						\
-             g_return_val_if_fail (n + sizeof(TYPESTD) <= len, -1);	\
+             g_return_val_if_fail (n + sizeof(TYPESTD) <= length, -1);	\
              t = va_arg (args, TYPESTD*);                           	\
              BEMEMCPY((char*) t, buffer, sizeof(TYPESTD));              \
              buffer += sizeof(TYPESTD);					\
@@ -733,71 +763,83 @@ gnet_vpack (const gchar* format, gchar* buffer, const guint len, va_list args)
 /* **************************************** */
 
 /**
- *  gnet_unpack:
- *  @format: Unpack format
- *  @buffer: Buffer to unpack from
- *  @len: Length of buffer
- *  @Varargs: Addresses of variables to unpack to
+ *  gnet_unpack
+ *  @format: unpack data format
+ *  @buffer: buffer to unpack from
+ *  @length: length of @buffer
+ *  @Varargs: addresses of variables to unpack to
  *
- *  The unpack format string is a list of types.  Each type is
- *  represented by a character.  Most types can be prefixed by an
- *  integer, which represents how many times it is repeated (eg,
- *  "4i2b" is equivalent to "iiiibb".
+ *  Reads the data in @buffer into @Varargs.  @format is a string that
+ *  describes the @Varargs and how they are to be packed.  This string
+ *  is a list of characters, each describing the type of an argument
+ *  in @Varargs.
+ *
+ *  An example:
+ *
+ *  <informalexample>
+ *  <programlisting>
+ *  char buf[5] = { 42, 0, 0, 0, 23 };
+ *  int myint;
+ *  int mybyte;
+ *  &space;
+ *  gnet_unpack("!ib", buf, sizeof(buf), &amp;myint, &amp;mybyte);
+ *  &comstart; Now myint is 42 and mybyte is 23 &comend;
+ *  </programlisting>
+ *  </informalexample>
  *
  *  In unpack, the arguments must be pointers to the appropriate type.
- *  Strings and byte arrays are allocated dynamicly (by g_new).  The
+ *  Strings and byte arrays are allocated dynamicly (by g_new()).  The
  *  caller is responsible for g_free()-ing it.
  *
- *  Native size/order is the default.  If the first character of
- *  FORMAT is < then little endian order and standard size are used.
- *  If the first character is > or !, then big endian (or network)
- *  order and standard size are used.  Standard sizes are 1 byte for
- *  chars, 2 bytes for shorts, and 4 bytes for ints and longs.
- * 
+ *  As a shortcut, most types can be prefixed by an integer to specify
+ *  how many times the type is repeated.  For example, "4i2b" is
+ *  equivalent to "iiiibb".  This repeat argument is refered below to
+ *  as REPEAT.
+ *
+ *  The types:
+ *
  *  x is a pad byte.  The byte is skipped and not stored.  We do not
  *  check its value.
  *
- *  b/B are signed/unsigned chars
+ *  b/B is a signed/unsigned char.
+ *
+ *  h/H is a signed/unsigned short.
+ *
+ *  i/I is a signed/unsigned int.
+ *
+ *  l/L is a signed/unsigned long.
+ *
+ *  f/D is a float/double (always native order/size).
+ *  
+ *  v is a void pointer (always native size).
+ *
+ *  s is a zero-terminated string.
  *
- *  h/H are signed/unsigned shorts (h is from sHort)
- * 
- *  i/I are signed/unsigned ints
- * 
- *  l/L are signed/unsigned longs
- * 
- *  f/D are floats/doubles (always native order/size)
- *    
- *  v is a void pointer (always native size)
- * 
- *  s is a zero-terminated string.  REPEAT is repeat.
- * 
  *  S is a zero-padded string of length REPEAT.  We read REPEAT
- *  characters or until a NULL character.  Any remaining characters are
- *  filled in with 0's.  REPEAT must be specified.
+ *  characters or until a NULL character.  Any remaining characters
+ *  are filled in with 0's.  REPEAT must be specified.
  * 
  *  r is a byte array of NEXT bytes.  NEXT is the next argument and is
- *  an integer.  REPEAT is repeat.  (r is from "raw")
+ *  a gint.  REPEAT is repeat.
  * 
  *  R is a byte array of REPEAT bytes.  REPEAT must be specified.
  * 
- *  String/byte array memory is allocated by unpack.
- * 
- *  Mnemonics: sHort, Integer, Float, Double, Pointer, String, Raw
- * 
- *  unpack was mostly inspired by Python's unpack, with some awareness
- *  of Perl's unpack.
- * 
- *  Returns: bytes unpacked; -1 if error.
+ *  p is a Pascal string.  The first byte read is the length of the
+ *  string and the string follows.  The unpacked string will be a
+ *  normal, NULL-terminated string.  REPEAT is repeat.
+ *
+ *  Returns: number of bytes unpacked; -1 on error.  The bytes are
+ *  unpacked to the variables pointed to by the @Varargs.
  * 
  **/
 gint 
-gnet_unpack (const gchar* format, gchar* buffer, guint len, ...)
+gnet_unpack (const gchar* format, gchar* buffer, gint length, ...)
 {
   va_list args;
   gint rv;
   
-  va_start (args, len);
-  rv = gnet_vunpack (format, buffer, len, args);
+  va_start (args, length);
+  rv = gnet_vunpack (format, buffer, length, args);
   va_end (args);
 
   return rv;
@@ -805,22 +847,22 @@ gnet_unpack (const gchar* format, gchar* buffer, guint len, ...)
 
 
 /**
- *  gnet_vunpack:
- *  @format: Unpack format (see below)
- *  @buffer: Buffer to unpack from
- *  @len: Length of buffer
+ *  gnet_vunpack
+ *  @format: unpack data format
+ *  @buffer: buffer to unpack from
+ *  @length: length of @buffer
  *  @args: var args
  *
- *  Var arg interface to gnet_unpack().  See gnet_unpack() for format
+ *  Var arg interface to gnet_unpack().  See gnet_unpack() for more
  *  information.
  *
- *  Returns: bytes packed; -1 if error.
+ *  Returns: number of bytes packed; -1 on error.
  *
  **/
 gint 
-gnet_vunpack (const gchar* format, gchar* buffer, guint len, va_list args)
+gnet_vunpack (const gchar* format, gchar* buffer, gint length, va_list args)
 {
-  guint n = 0;
+  gint n = 0;
   gchar* p = (gchar*) format;
   guint mult = 0;
   gint sizemode = 0;	/* 1 = little, 2 = big */
@@ -843,7 +885,7 @@ gnet_vunpack (const gchar* format, gchar* buffer, guint len, va_list args)
 	case 'x': 
 	  {	
 	    mult = mult? mult:1;
-	    g_return_val_if_fail (n + mult <= len, FALSE);
+	    g_return_val_if_fail (n + mult <= length, FALSE);
 
 	    buffer += mult;
 	    n += mult;
@@ -874,13 +916,13 @@ gnet_vunpack (const gchar* format, gchar* buffer, guint len, va_list args)
 	    for (mult=(mult?mult:1); mult; --mult)
 	      {
 		gchar** sp; 
-		guint slen;
+		gsize slen;
 
 		sp = va_arg (args, gchar**);
 		g_return_val_if_fail (sp, -1);
 
-		slen = strlenn(buffer, len - n);
-		g_return_val_if_fail (n + slen <= len, FALSE);
+		slen = strlenn(buffer, length - n);
+		g_return_val_if_fail (n + slen <= length, FALSE);
 
 		*sp = g_new(gchar, slen + 1);
 		memcpy (*sp, buffer, slen);
@@ -896,19 +938,20 @@ gnet_vunpack (const gchar* format, gchar* buffer, guint len, va_list args)
 	case 'S':
 	  { 
 	    gchar** sp; 
-	    guint slen;
+	    gsize slen;
 
 	    g_return_val_if_fail (mult, -1);
 
 	    sp = va_arg (args, gchar**);
 	    g_return_val_if_fail (sp, -1);
 
-	    slen = MIN(mult, strlenn(buffer, len - n));
-	    g_return_val_if_fail (n + slen <= len, -1);
+	    slen = MIN(mult, strlenn(buffer, length - n));
+	    g_return_val_if_fail ((n + slen) <= length, -1);
 
 	    *sp = g_new(gchar, mult + 1);
+
 	    memcpy (*sp, buffer, slen);
-	    while (slen < mult + 1) (*sp)[slen++] = 0;
+	    while (slen < (mult + 1)) (*sp)[slen++] = '\0';
 	    buffer += mult;
 	    n += mult;
 
@@ -926,7 +969,7 @@ gnet_vunpack (const gchar* format, gchar* buffer, guint len, va_list args)
 		ln = va_arg (args, guint);
 
 		g_return_val_if_fail (sp, -1);
-		g_return_val_if_fail (n + ln <= len, FALSE);
+		g_return_val_if_fail (n + ln <= length, FALSE);
 
 		*sp = g_new(char, ln);
 		memcpy(*sp, buffer, ln);
@@ -945,7 +988,7 @@ gnet_vunpack (const gchar* format, gchar* buffer, guint len, va_list args)
 	    g_return_val_if_fail (sp, -1);
 
 	    g_return_val_if_fail (mult, -1);
-	    g_return_val_if_fail (n + mult <= len, -1);
+	    g_return_val_if_fail (n + mult <= length, -1);
 
 	    *sp = g_new(char, mult);
 	    memcpy(*sp, buffer, mult);
@@ -964,11 +1007,11 @@ gnet_vunpack (const gchar* format, gchar* buffer, guint len, va_list args)
 
 		sp = va_arg (args, gchar**);
 		g_return_val_if_fail (sp, -1);
-		g_return_val_if_fail (n + 1 <= len, FALSE);
+		g_return_val_if_fail (n + 1 <= length, FALSE);
 
 		slen = *buffer++; 
 		++n;
-		g_return_val_if_fail (n + slen <= len, FALSE);
+		g_return_val_if_fail (n + slen <= length, FALSE);
 
 		*sp = g_new(gchar, slen + 1);
 		memcpy (*sp, buffer, slen); 
