@@ -22,21 +22,6 @@
 #include "socks-private.h"
 
 
-static int socks_get_version (void);
-
-/* **************************************** */
-
-static int
-socks_get_version (void)
-{
-  const gchar* verc;
-
-  if ((verc = g_getenv("SOCKS_VERSION"))) 
-    return atoi(verc);
-
-  return GNET_DEFAULT_SOCKS_VERSION;
-}
-
 
 /* **************************************** */
 
@@ -88,7 +73,7 @@ struct async_data
   gpointer data;
 };
 
-static void async_cb (GTcpSocket* socket, GTcpSocketNewAsyncStatus status, gpointer data);
+static void async_cb (GTcpSocket* socket, gpointer data);
 
 
 GTcpSocketNewAsyncID
@@ -123,11 +108,11 @@ gnet_private_socks_tcp_socket_new_async (const GInetAddr* addr,
 
 
 static void
-async_cb (GTcpSocket* socket, GTcpSocketNewAsyncStatus status, gpointer data)
+async_cb (GTcpSocket* socket, gpointer data)
 {
   struct async_data* ad = (struct async_data*) data;
   
-  if (status == GTCP_SOCKET_NEW_ASYNC_STATUS_OK)
+  if (socket != NULL)
     {
       int rv;
 
@@ -135,14 +120,14 @@ async_cb (GTcpSocket* socket, GTcpSocketNewAsyncStatus status, gpointer data)
       if (rv < 0)
 	goto error;
 
-      (ad->func)(socket, GTCP_SOCKET_NEW_ASYNC_STATUS_OK, ad->data);
+      (ad->func)(socket, ad->data);
       gnet_inetaddr_delete (ad->addr);
       g_free (ad);
       return;
     }
 
  error:
-  (ad->func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, ad->data);
+  (ad->func)(NULL, ad->data);
   gnet_inetaddr_delete (ad->addr);
   g_free (ad);
 }
@@ -157,15 +142,14 @@ socks_negotiate_connect (GTcpSocket *s, const GInetAddr *dst)
   GIOChannel *ioc;
   int ver, ret;
 
-  ioc = gnet_tcp_socket_get_iochannel(s);
-  ver = socks_get_version();
+  ioc = gnet_tcp_socket_get_io_channel(s);
+  ver = gnet_socks_get_version();
   if (ver == 5)
     ret = socks5_negotiate_connect (ioc, dst);
   else if (ver == 4)
     ret = socks4_negotiate_connect (ioc, dst);
   else
     ret = -1;
-  g_io_channel_unref(ioc);
 
   return ret;
 }
@@ -255,7 +239,7 @@ gnet_private_socks_tcp_socket_server_new (gint port)
   int			rv;
 
   /* We only support SOCKS 5 */
-  if (socks_get_version () != 5)
+  if (gnet_socks_get_version () != 5)
     return NULL;
 
   /* Get SOCKS server */
@@ -289,7 +273,7 @@ socks5_negotiate_bind (GTcpSocket* socket, int port)
   struct socks5_h s5h;
   int len;
 
-  ioc = gnet_tcp_socket_get_iochannel(socket);
+  ioc = gnet_tcp_socket_get_io_channel(socket);
 
   s5r[0] = 5;
   s5r[1] = 1;	/* no authentication */
@@ -326,7 +310,6 @@ socks5_negotiate_bind (GTcpSocket* socket, int port)
   return 0;
 
  error:
-  g_io_channel_unref(ioc);
   return -1;
 }
 
@@ -350,9 +333,8 @@ gnet_private_socks_tcp_socket_server_accept (GTcpSocket* socket)
   server_port = g_ntohs(GNET_SOCKADDR_IN(socket->sa).sin_port);
 
   /* this reply reveals the connecting hosts ip and port */
-  iochannel = gnet_tcp_socket_get_iochannel(socket);
+  iochannel = gnet_tcp_socket_get_io_channel(socket);
   error = gnet_io_channel_readn(iochannel, (gchar*) &s5h, 10, &len);
-  g_io_channel_unref (iochannel);
   if (error != G_IO_ERROR_NONE)
     return NULL;
 
@@ -390,13 +372,12 @@ gnet_private_socks_tcp_socket_server_accept (GTcpSocket* socket)
       GIOChannel* iochannel;
 
       /* This will recreate the IOChannel */
-      iochannel = gnet_tcp_socket_get_iochannel (socket);
+      iochannel = gnet_tcp_socket_get_io_channel (socket);
 
       /* Set the watch on the new IO channel */
       socket->accept_watch = g_io_add_watch(iochannel, 
 					    G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL, 
 					    socks_tcp_socket_server_accept_async_cb, socket);
-      g_io_channel_unref (iochannel);
     }
 
   return s;
@@ -419,11 +400,10 @@ gnet_private_socks_tcp_socket_server_accept_async (GTcpSocket* socket,
   socket->accept_data = user_data;
 
   /* Add read watch */
-  iochannel = gnet_tcp_socket_get_iochannel (socket);
+  iochannel = gnet_tcp_socket_get_io_channel (socket);
   socket->accept_watch = g_io_add_watch(iochannel, 
 					G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL, 
 					socks_tcp_socket_server_accept_async_cb, socket);
-  g_io_channel_unref (iochannel);
 }
 
 
