@@ -1,5 +1,5 @@
 /* GNet - Networking library
- * Copyright (C) 2000  David Helder
+ * Copyright (C) 2000-3  David Helder
  * Copyright (C) 2000  Andrew Lanoix
  *
  * This library is free software; you can redistribute it and/or
@@ -23,54 +23,61 @@
 #include "tcp.h"
 
 /**
- *  gnet_tcp_socket_connect:
- *  @hostname: Name of host to connect to.
- *  @port: Port to connect to.
+ *  gnet_tcp_socket_connect
+ *  @hostname: host name
+ *  @port: port
  *
- *  A quick and easy #GTcpSocket constructor.  This connects to the
- *  specified address and port.  This function does block
- *  (gnet_tcp_socket_connect_async() does not).  Use this function
- *  when you're a client connecting to a server and you don't mind
- *  blocking and don't want to mess with #GInetAddr's.  You can get
- *  the InetAddr of the socket by calling
- *  gnet_tcp_socket_get_inetaddr().
+ *  Creates a #GTcpSocket and connects to @hostname:@port.  This
+ *  function blocks (while gnet_tcp_socket_connect_async() does not).
+ *  To get the #GInetAddr of the #GTcpSocket, call
+ *  gnet_tcp_socket_get_remote_inetaddr().
  *
- *  Returns: A new #GTcpSocket, or NULL if there was a failure.
+ *  Returns: a new #GTcpSocket; NULL on error.
  **/
 GTcpSocket*
 gnet_tcp_socket_connect (const gchar* hostname, gint port)
 {
-  GInetAddr* ia;
+  GList* ia_list;
+  GList* i;
   GTcpSocket* socket;
 
-  ia = gnet_inetaddr_new(hostname, port);
-  if (ia == NULL)
+  ia_list = gnet_inetaddr_new_list (hostname, port);
+  if (ia_list == NULL)
     return NULL;
 
-  socket = gnet_tcp_socket_new(ia);
-  gnet_inetaddr_delete(ia);
+  socket = NULL;
+  for (i = ia_list; i != NULL; i = i->next)
+    {
+      GInetAddr* ia;
+
+      ia = (GInetAddr*) i->data;
+      socket = gnet_tcp_socket_new (ia);
+      if (socket)
+	break;
+    }
+  for (i = ia_list; i != NULL; i = i->next)
+    gnet_inetaddr_delete ((GInetAddr*) i->data);
+  g_list_free (ia_list);
 
   return socket;
 }
 
 
 /**
- *  gnet_tcp_socket_connect_async:
- *  @hostname: Name of host to connect to
- *  @port: Port to connect to
- *  @func: Callback function
- *  @data: User data passed when callback function is called.
+ *  gnet_tcp_socket_connect_async
+ *  @hostname: host name
+ *  @port: port
+ *  @func: callback function
+ *  @data: data to pass to @func on callback
  *
- *  A quick and easy asynchronous #GTcpSocket constructor.  This
- *  connects to the specified address and port and then calls the
- *  callback with the data.  Use this function when you're a client
- *  connecting to a server and you don't want to block or mess with
- *  #GInetAddr's.  It will call the callback if there is a failure.
- *  It will never call the callback before the function returns.
+ *  Asynchronously creates a #GTcpSocket and connects to
+ *  @hostname:@port.  The callback is called when the connection is
+ *  made or an error occurs.  The callback will not be called during
+ *  the call to this function.
  *
- *  Returns: ID of the connection which can be used with
- *  gnet_tcp_socket_connect_async_cancel() to cancel it; NULL on
- *  failure.
+ *  Returns: the ID of the connection; NULL on failure.  The ID can be
+ *  used with gnet_tcp_socket_connect_async_cancel() to cancel the
+ *  connection.
  *
  **/
 GTcpSocketConnectAsyncID
@@ -88,12 +95,12 @@ gnet_tcp_socket_connect_async (const gchar* hostname, gint port,
   state->data = data;
 
   state->inetaddr_id = 
-    gnet_inetaddr_new_async (hostname, port,  
-			     gnet_tcp_socket_connect_inetaddr_cb, 
-			     state);
+    gnet_inetaddr_new_list_async (hostname, port,  
+				  gnet_tcp_socket_connect_inetaddr_cb, 
+				  state);
 
-  /* On failure, gnet_inetaddr_new_async() returns NULL.  It will not
-     call the callback before it returns. */
+  /* On failure, gnet_inetaddr_new_list_async() returns NULL.  It will
+     not call the callback before it returns. */
   if (state->inetaddr_id == NULL)
     {
       g_free (state);
@@ -106,66 +113,113 @@ gnet_tcp_socket_connect_async (const gchar* hostname, gint port,
 
 
 void
-gnet_tcp_socket_connect_inetaddr_cb (GInetAddr* inetaddr, 
-				     GInetAddrAsyncStatus status, 
-				     gpointer data)
+gnet_tcp_socket_connect_inetaddr_cb (GList* ia_list, gpointer data)
 {
   GTcpSocketConnectState* state = (GTcpSocketConnectState*) data;
 
-  if (status == GINETADDR_ASYNC_STATUS_OK)
+  if (ia_list != NULL) /* Success */
     {
-      gpointer tcp_id;
-
-      state->ia = gnet_inetaddr_clone (inetaddr);
+      g_assert (ia_list);
 
       state->inetaddr_id = NULL;
+      state->ia_list = ia_list;
+      state->ia_next = ia_list;
 
-      tcp_id = gnet_tcp_socket_new_async (inetaddr, 
-					  gnet_tcp_socket_connect_tcp_cb, 
-					  state);
-      /* gnet_tcp_socket_new_async() may call the callback before it
-	 returns.  state may have been deleted. */
-      if (tcp_id)
-	state->tcp_id = tcp_id;
-    }
-  else
-    {
-      (*state->func)(NULL, NULL, 
-		     GTCP_SOCKET_CONNECT_ASYNC_STATUS_INETADDR_ERROR, 
+      while (state->ia_next != NULL)
+	{
+	  GInetAddr* ia;
+	  gpointer tcp_id;
+
+	  ia = (GInetAddr*) state->ia_next->data;
+	  state->ia_next = state->ia_next->next;
+
+	  tcp_id = gnet_tcp_socket_new_async (ia, 
+					      gnet_tcp_socket_connect_tcp_cb, 
+					      state);
+	  if (tcp_id)	/* Success */
+	    {
+	      state->tcp_id = tcp_id;
+	      return;
+	    }
+	}
+
+      /* Failure: We could not async connect to any address.
+         In practice, new_async() rarely fails immediately.  */
+      state->in_callback = TRUE;
+      (*state->func)(NULL, GTCP_SOCKET_CONNECT_ASYNC_STATUS_INETADDR_ERROR, 
 		     state->data);
-      g_free(state);
+      state->in_callback = FALSE;
+
+      gnet_tcp_socket_connect_async_cancel (state);
+    }
+  else /* Failure */
+    {
+      state->in_callback = TRUE;
+      (*state->func)(NULL, GTCP_SOCKET_CONNECT_ASYNC_STATUS_INETADDR_ERROR, 
+		     state->data);
+      state->in_callback = FALSE;
+
+      gnet_tcp_socket_connect_async_cancel (state);
     }
 }
 
 
 void 
-gnet_tcp_socket_connect_tcp_cb (GTcpSocket* socket, 
-				GTcpSocketConnectAsyncStatus status, 
-				gpointer data)
+gnet_tcp_socket_connect_tcp_cb (GTcpSocket* socket, gpointer data)
 {
   GTcpSocketConnectState* state = (GTcpSocketConnectState*) data;
 
-  if (status == GTCP_SOCKET_NEW_ASYNC_STATUS_OK)
+  g_return_if_fail (state != NULL);
+
+  state->tcp_id = NULL;
+
+  /* Success */
+  if (socket != NULL)
     {
-      (*state->func)(socket, state->ia, 
-		     GTCP_SOCKET_CONNECT_ASYNC_STATUS_OK, state->data);
-    }
-  else
-    {
-      (*state->func)(NULL, NULL, 
-		     GTCP_SOCKET_CONNECT_ASYNC_STATUS_TCP_ERROR, state->data);
-      gnet_inetaddr_delete (state->ia);
+      state->in_callback = TRUE;
+      (*state->func)(socket, GTCP_SOCKET_CONNECT_ASYNC_STATUS_OK, state->data);
+      state->in_callback = FALSE;
+
+      gnet_tcp_socket_connect_async_cancel (state);
+
+      return;
     }
 
-  g_free (state);
+  /* Failure: Could not connect to address */
+
+  /* Try other addresses */
+  while (state->ia_next != NULL)
+    {
+      GInetAddr* ia;
+      gpointer tcp_id = NULL;
+
+      ia = (GInetAddr*) state->ia_next->data;
+      state->ia_next = state->ia_next->next;
+
+      tcp_id = gnet_tcp_socket_new_async (ia, 
+					  gnet_tcp_socket_connect_tcp_cb, 
+					  state);
+      if (tcp_id)	/* Success */
+	{
+	  state->tcp_id = tcp_id;
+	  return;
+	}
+    }
+
+  /* Failure: No more addresses */
+  state->in_callback = TRUE;
+  (*state->func)(NULL, GTCP_SOCKET_CONNECT_ASYNC_STATUS_TCP_ERROR, state->data);
+  state->in_callback = FALSE;
+
+  gnet_tcp_socket_connect_async_cancel (state);
 }
 
 
 /**
- *  gnet_tcp_socket_connect_async_cancel:
- *  @id: ID of the connection.
+ *  gnet_tcp_socket_connect_async_cancel
+ *  @id: ID of the connection
  *
- *  Cancel an asynchronous connection that was started with
+ *  Cancels an asynchronous connection that was started with
  *  gnet_tcp_socket_connect_async().
  * 
  */
@@ -176,17 +230,28 @@ gnet_tcp_socket_connect_async_cancel (GTcpSocketConnectAsyncID id)
 
   g_return_if_fail (state != NULL);
 
+  /* Ignore if user called in the middle of a callback */
+  if (state->in_callback)
+    return;
+
+  if (state->ia_list)
+    {
+      GList* i;
+      
+      for (i = state->ia_list; i != NULL; i = i->next)
+	gnet_inetaddr_delete ((GInetAddr*) i->data);
+      g_list_free (state->ia_list);
+    }
+
   if (state->inetaddr_id)
     {
       gnet_inetaddr_new_async_cancel(state->inetaddr_id);
     }
-  else if (state->tcp_id)
+
+  if (state->tcp_id)
     {
-      gnet_inetaddr_delete (state->ia);
       gnet_tcp_socket_new_async_cancel (state->tcp_id);
     }
-  else
-    g_assert_not_reached();
 
   g_free (state);
 }
@@ -196,14 +261,13 @@ gnet_tcp_socket_connect_async_cancel (GTcpSocketConnectAsyncID id)
 
 
 /**
- *  gnet_tcp_socket_new:
- *  @addr: Address to connect to.
+ *  gnet_tcp_socket_new
+ *  @addr: address
  *
- *  Connect to a specified address.  Use this sort of socket when
- *  you're a client connecting to a server.  This function will block
- *  to connect.  SOCKS is used if SOCKS is enabled.
+ *  Creates a #GTcpSocket and connects to @addr.  This function
+ *  blocks.  SOCKS is used if SOCKS is enabled.
  *
- *  Returns a new #GTcpSocket, or NULL if there was a failure.
+ *  Returns: a new #GTcpSocket; NULL on error.
  *
  **/
 GTcpSocket* 
@@ -223,13 +287,13 @@ gnet_tcp_socket_new (const GInetAddr* addr)
 
 /**
  *  gnet_tcp_socket_new_direct:
- *  @addr: Address to connect to.
+ *  @addr: address
  *
- *  Connect directly to a specified address and do not use SOCKS even
- *  if SOCKS is enabled.  Most users should use gnet_tcp_socket_new().
- *  This is used internally to implement SOCKS.
+ *  Creates a #GTcpSocket and connects to @addr without using SOCKS.
+ *  This function blocks.  Most users should use
+ *  gnet_tcp_socket_new().
  *
- *  Returns a new #GTcpSocket, or NULL if there was a failure.
+ *  Returns: a new #GTcpSocket; NULL on error.
  *
  **/
 GTcpSocket* 
@@ -237,13 +301,12 @@ gnet_tcp_socket_new_direct (const GInetAddr* addr)
 {
   int 			sockfd;
   GTcpSocket* 		s;
-  struct sockaddr_in* 	sa_in;
   int			rv;
 
   g_return_val_if_fail (addr != NULL, NULL);
 
   /* Create socket */
-  sockfd = socket (AF_INET, SOCK_STREAM, 0);
+  sockfd = socket (GNET_INETADDR_FAMILY(addr), SOCK_STREAM, 0);
   if (sockfd < 0)
     return NULL;
 
@@ -251,14 +314,11 @@ gnet_tcp_socket_new_direct (const GInetAddr* addr)
   s = g_new0 (GTcpSocket, 1);
   s->sockfd = sockfd;
   s->ref_count = 1;
-
-  /* Set up address and port for connection */
-  memcpy(&s->sa, &addr->sa, sizeof(s->sa));
-  sa_in = (struct sockaddr_in*) &s->sa;
-  sa_in->sin_family = AF_INET;
+  s->sa = addr->sa;
 
   /* Connect */
-  rv = connect(sockfd, &s->sa, sizeof(s->sa));
+  rv = connect(sockfd, 
+	       &GNET_SOCKADDR_SA(s->sa), GNET_SOCKADDR_LEN(s->sa));
   if (rv != 0)
     {
       GNET_CLOSE_SOCKET(s->sockfd);
@@ -274,20 +334,21 @@ gnet_tcp_socket_new_direct (const GInetAddr* addr)
 
 
 /**
- *  gnet_tcp_socket_new_async:
- *  @addr: Address to connect to.
- *  @func: Callback function.
- *  @data: User data passed when callback function is called.
+ *  gnet_tcp_socket_new_async
+ *  @addr: address
+ *  @func: callback function
+ *  @data: data to pass to @func on callback
  *
- *  Connect to a specifed address asynchronously.  When the connection
- *  is complete or there is an error, it will call the callback.  It
- *  may call the callback before the function returns.  It will call
- *  the callback if there is a failure.  SOCKS is used if SOCKS is
- *  enabled.  The SOCKS negotiation will block.
+ *  Asynchronously creates a #GTcpSocket and connects to @addr.  The
+ *  callback is called once the connection is made or an error occurs.
+ *  The callback will not be called during the call to this function.
  *
- *  Returns: ID of the connection which can be used with
- *  gnet_tcp_socket_connect_async_cancel() to cancel it; NULL on
- *  failure.
+ *  SOCKS is used if SOCKS is enabled.  The SOCKS negotiation will
+ *  block.
+ *
+ *  Returns: the ID of the connection; NULL on failure.  The ID can be
+ *  used with gnet_tcp_socket_new_async_cancel() to cancel the
+ *  connection.
  *
  **/
 GTcpSocketNewAsyncID
@@ -304,8 +365,6 @@ gnet_tcp_socket_new_async (const GInetAddr* addr,
 
   /* Otherwise, connect directly to the address */
   return gnet_tcp_socket_new_async_direct (addr, func, data);
-
-
 }
 
 
@@ -313,15 +372,20 @@ gnet_tcp_socket_new_async (const GInetAddr* addr,
 
 
 /**
- *  gnet_tcp_socket_new_async_direct:
- *  @addr: Address to connect to.
+ *  gnet_tcp_socket_new_async_direct
+ *  @addr: address
+ *  @func: callback function
+ *  @data: data to pass to @func on callback
  *
- *  Connect directly to a specified address asynchronously and do not
- *  use SOCKS even if SOCKS is enabled.  Most users should use
- *  gnet_tcp_socket_new_async().  This is used internally to implement
- *  SOCKS.
+ *  Asynchronously creates a #GTcpSocket and connects to @addr without
+ *  using SOCKS.  Most users should use gnet_tcp_socket_new_async()
+ *  instead.  The callback is called once the connection is made or an
+ *  error occurs.  The callback will not be called during the call to
+ *  this function.
  *
- *  Returns a new #GTcpSocket, or NULL if there was a failure.
+ *  Returns: the ID of the connection; NULL on failure.  The ID can be
+ *  used with gnet_tcp_socket_new_async_cancel() to cancel the
+ *  connection.
  *
  **/
 GTcpSocketNewAsyncID
@@ -332,60 +396,42 @@ gnet_tcp_socket_new_async_direct (const GInetAddr* addr,
   gint 			sockfd;
   gint 			flags;
   GTcpSocket* 		s;
-  struct sockaddr	sa;
-  struct sockaddr_in* 	sa_in;
   GTcpSocketAsyncState* state;
 
   g_return_val_if_fail(addr != NULL, NULL);
   g_return_val_if_fail(func != NULL, NULL);
 
   /* Create socket */
-  sockfd = socket(AF_INET, SOCK_STREAM, 0);
+  sockfd = socket(GNET_INETADDR_FAMILY(addr), SOCK_STREAM, 0);
   if (sockfd < 0)
-    {
-      (func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, data);
-      return NULL;
-    }
+    return NULL;
 
   /* Get the flags (should all be 0?) */
   flags = fcntl(sockfd, F_GETFL, 0);
   if (flags == -1)
-    {
-      (func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, data);
-      return NULL;
-    }
+    return NULL;
 
   if (fcntl(sockfd, F_SETFL, flags | O_NONBLOCK) == -1)
-    {
-      (func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, data);
-      return NULL;
-    }
+    return NULL;
 
   /* Create our structure */
   s = g_new0(GTcpSocket, 1);
   s->ref_count = 1;
   s->sockfd = sockfd;
 
-  /* Set up address and port for connection */
-  memcpy(&sa, &addr->sa, sizeof(sa));
-  sa_in = (struct sockaddr_in*) &sa;
-  sa_in->sin_family = AF_INET;
-
   /* Connect (but non-blocking!) */
-  if (connect(s->sockfd, &sa, sizeof(s->sa)) < 0)
+  if (connect(s->sockfd, &GNET_INETADDR_SA(addr), 
+	      GNET_INETADDR_LEN(addr)) < 0)
     {
       if (errno != EINPROGRESS)
 	{
-	  (func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, data);
 	  g_free(s);
 	  return NULL;
 	}
     }
 
   /* Save address */ 
-  memcpy(&s->sa, &addr->sa, sizeof(s->sa));
-  sa_in = (struct sockaddr_in*) &sa;
-  sa_in->sin_family = AF_INET;
+  s->sa = addr->sa;
 
   /* Note that if connect returns 0, then we're already connected and
      we could call the call back immediately.  But, it would probably
@@ -398,7 +444,7 @@ gnet_tcp_socket_new_async_direct (const GInetAddr* addr,
   state->func = func;
   state->data = data;
   state->flags = flags;
-  state->iochannel = gnet_private_iochannel_new(s->sockfd);
+  state->iochannel = gnet_private_io_channel_new(s->sockfd);
   state->connect_watch = g_io_add_watch(state->iochannel,
 					GNET_ANY_IO_CONDITION,
 					gnet_tcp_socket_new_async_cb, 
@@ -440,13 +486,13 @@ gnet_tcp_socket_new_async_cb (GIOChannel* iochannel,
     goto error;
 
   /* Success */
-  (*state->func)(state->socket, GTCP_SOCKET_NEW_ASYNC_STATUS_OK, state->data);
+  (*state->func)(state->socket, state->data);
   g_free(state);
   return FALSE;
 
   /* Error */
  error:
-  (*state->func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, state->data);
+  (*state->func)(NULL, state->data);
   gnet_tcp_socket_delete (state->socket);
   g_free(state);
 
@@ -455,10 +501,10 @@ gnet_tcp_socket_new_async_cb (GIOChannel* iochannel,
 
 
 /**
- *  gnet_tcp_socket_new_async_cancel:
- *  @id: ID of the connection.
+ *  gnet_tcp_socket_new_async_cancel
+ *  @id: ID of the connection
  *
- *  Cancel an asynchronous connection that was started with
+ *  Cancels an asynchronous #GTcpSocket creation that was started with
  *  gnet_tcp_socket_new_async().
  *
  **/
@@ -486,8 +532,6 @@ gnet_tcp_socket_new_async_direct (const GInetAddr* addr,
   gint sockfd;
   gint status;
   GTcpSocket* s;
-  struct sockaddr sa;
-  struct sockaddr_in* sa_in;
   GTcpSocketAsyncState* state;
   u_long arg;
 
@@ -495,43 +539,33 @@ gnet_tcp_socket_new_async_direct (const GInetAddr* addr,
   g_return_val_if_fail(func != NULL, NULL);
 
   /* Create socket */
-  sockfd = socket(AF_INET, SOCK_STREAM, 0);
+  sockfd = socket(GNET_INETADDR_FAMILY(addr), SOCK_STREAM, 0);
   if (sockfd == INVALID_SOCKET)
-    {
-      (func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, data);
       return NULL;
-    }
 	
   /* Create our structure */
   s = g_new0(GTcpSocket, 1);
   s->ref_count = 1;
   s->sockfd = sockfd;
 
-  /* Set up address and port for connection */
-  memcpy(&sa, &addr->sa, sizeof(sa));
-  sa_in = (struct sockaddr_in*) &sa;
-  sa_in->sin_family = AF_INET;
-
   /* Force the socket into non-blocking mode */
   arg = 1;
   ioctlsocket(sockfd, FIONBIO, &arg);
 
-  status = connect(s->sockfd, &sa, sizeof(s->sa));
+  status = connect(s->sockfd, &GNET_INETADDR_SA(addr), 
+		   GNET_INETADDR_LEN(addr));
   if (status == SOCKET_ERROR) /* Returning an error is ok, unless.. */
     {
       status = WSAGetLastError();
       if (status != WSAEWOULDBLOCK)
 	{
-	  (func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, data);
 	  g_free(s);
 	  return NULL;
 	}
     }
 
   /* Save address */ 
-  memcpy(&s->sa, &addr->sa, sizeof(s->sa));
-  sa_in = (struct sockaddr_in*) &sa;
-  sa_in->sin_family = AF_INET;
+  s->sa = addr->sa;
 
   /* Wait for the connection */
   state = g_new0(GTcpSocketAsyncState, 1);
@@ -541,17 +575,13 @@ gnet_tcp_socket_new_async_direct (const GInetAddr* addr,
   state->socket->sockfd = sockfd;
 
   state->connect_watch = 
-    g_io_add_watch(gnet_private_iochannel_new(s->sockfd),
+    g_io_add_watch(gnet_private_io_channel_new(s->sockfd),
 		   G_IO_IN | G_IO_ERR,
 		   gnet_tcp_socket_new_async_cb, 
 		   state);
 
-
   if (state->connect_watch <= 0)
-    {
-      (func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, data);
       return NULL;
-    }
 
   return state;
 }
@@ -568,12 +598,12 @@ gnet_tcp_socket_new_async_cb (GIOChannel* iochannel,
   if (condition & G_IO_ERR)
     goto error;
 
-  (*state->func)(state->socket, GTCP_SOCKET_NEW_ASYNC_STATUS_OK, state->data);
+  (*state->func)(state->socket, state->data);
   g_free (state);
   return FALSE;
 
  error:
-  (*state->func)(NULL, GTCP_SOCKET_NEW_ASYNC_STATUS_ERROR, state->data);
+  (*state->func)(NULL, state->data);
   gnet_tcp_socket_delete (state->socket);
   g_free (state);
   return FALSE;
@@ -595,155 +625,188 @@ gnet_tcp_socket_new_async_cancel (GTcpSocketNewAsyncID id)
 
 
 /**
- *  gnet_tcp_socket_delete:
- *  @s: TcpSocket to delete.
+ *  gnet_tcp_socket_delete
+ *  @socket: a #GTcpSocket
  *
- *  Close and delete a #GTcpSocket.
+ *  Deletes a #GTcpSocket.
  *
  **/
 void
-gnet_tcp_socket_delete(GTcpSocket* s)
+gnet_tcp_socket_delete (GTcpSocket* socket)
 {
-  if (s != NULL)
-    gnet_tcp_socket_unref(s);
+  if (socket != NULL)
+    gnet_tcp_socket_unref (socket);
 }
 
 
 
 /**
  *  gnet_tcp_socket_ref
- *  @s: #GTcpSocket to reference
+ *  @socket: a #GTcpSocket
  *
- *  Increment the reference counter of the GTcpSocket.
+ *  Adds a reference to a #GTcpSocket.
  *
  **/
 void
-gnet_tcp_socket_ref(GTcpSocket* s)
+gnet_tcp_socket_ref (GTcpSocket* socket)
 {
-  g_return_if_fail(s != NULL);
+  g_return_if_fail (socket != NULL);
 
-  ++s->ref_count;
+  ++socket->ref_count;
 }
 
 
 /**
  *  gnet_tcp_socket_unref
- *  @s: #GTcpSocket to unreference
+ *  @socket: a #GTcpSocket to unreference
  *
- *  Remove a reference from the #GTcpSocket.  When reference count
- *  reaches 0, the socket is deleted.
+ *  Removes a reference from a #GTcpSocket.  A #GTcpSocket is deleted
+ *  when the reference count reaches 0.
  *
  **/
 void
-gnet_tcp_socket_unref(GTcpSocket* s)
+gnet_tcp_socket_unref (GTcpSocket* socket)
 {
-  g_return_if_fail(s != NULL);
+  g_return_if_fail (socket != NULL);
 
-  --s->ref_count;
+  socket->ref_count--;
 
-  if (s->ref_count == 0)
+  if (socket->ref_count == 0)
     {
-      if (s->accept_watch)
-	g_source_remove (s->accept_watch);
+      if (socket->accept_watch)
+	g_source_remove (socket->accept_watch);
 
-      GNET_CLOSE_SOCKET(s->sockfd);	/* Don't care if this fails... */
+      GNET_CLOSE_SOCKET(socket->sockfd);/* Don't care if this fails... */
 
-      if (s->iochannel)
-	g_io_channel_unref(s->iochannel);
+      if (socket->iochannel)
+	g_io_channel_unref (socket->iochannel);
 
-      g_free(s);
+      g_free(socket);
     }
 }
 
 
 
 /**
- *  gnet_tcp_socket_get_iochannel:
- *  @socket: GTcpSocket to get GIOChannel from.
+ *  gnet_tcp_socket_get_io_channel
+ *  @socket: a #GTcpSocket
  *
- *  Get the #GIOChannel for the #GTcpSocket.
+ *  Gets the #GIOChannel of a #GTcpSocket.
  *
  *  For a client socket, the #GIOChannel represents the data stream.
  *  Use it like you would any other #GIOChannel.
  *
- *  For a server socket however, the #GIOChannel represents incoming
- *  connections.  If you can read from it, there's a connection
- *  waiting.
+ *  For a server socket however, the #GIOChannel represents the
+ *  listening socket.  When it's readable, there's a connection
+ *  waiting to be accepted.  However, using
+ *  gnet_tcp_socket_server_accept_async() is more elegant than
+ *  watching the #GIOChannel.
  *
- *  There is one channel for every socket.  This function refs the
- *  channel before returning it.  You should unref the channel when
- *  you are done with it.  However, you should not close the channel -
- *  this is done when you delete the socket.
+ *  Every #GTcpSocket has one and only one #GIOChannel.  If you ref
+ *  the channel, then you must unref it eventually.  Do not close the
+ *  channel.  The channel is closed by GNet when the socket is
+ *  deleted.
  *
- *  Returns: A #GIOChannel; NULL on failure.
+ *  Returns: a #GIOChannel.
  *
  **/
 GIOChannel* 
-gnet_tcp_socket_get_iochannel(GTcpSocket* socket)
+gnet_tcp_socket_get_io_channel (GTcpSocket* socket)
 {
   g_return_val_if_fail (socket != NULL, NULL);
 
   if (socket->iochannel == NULL)
-    socket->iochannel = gnet_private_iochannel_new(socket->sockfd);
-   
-  g_io_channel_ref (socket->iochannel);
+    socket->iochannel = gnet_private_io_channel_new(socket->sockfd);
 
   return socket->iochannel;
 }
 
 
-
 /**
- *  gnet_tcp_socket_get_inetaddr:
- *  @socket: #GTcpSocket to get address of.
+ *  gnet_tcp_socket_get_remote_inetaddr
+ *  @socket: a #GTcpSocket
  *
- *  Get the address of the socket.  If the socket is client socket,
- *  the address is the address of the remote host it is connected to.
- *  If the socket is a server socket, the address is the address of
- *  the local host.  (Though you should use
- *  gnet_inetaddr_gethostaddr() to get the #GInetAddr of the local
- *  host.)
+ *  Gets the address of the remote host from a #GTcpSocket.  This
+ *  function does not work on server sockets.
  *
- *  Returns: #GInetAddr of socket; NULL on failure.
+ *  Returns: a #GInetAddr.
  *
  **/
 GInetAddr* 
-gnet_tcp_socket_get_inetaddr(const GTcpSocket* socket)
+gnet_tcp_socket_get_remote_inetaddr (const GTcpSocket* socket)
 {
+  GInetAddr* ia;
+
   g_return_val_if_fail (socket != NULL, NULL);
 
-  return gnet_private_inetaddr_sockaddr_new(socket->sa);
+  ia = g_new0(GInetAddr, 1);
+  ia->sa = socket->sa;
+  ia->ref_count = 1;
+
+  return ia;
 }
 
 
 /**
- *  gnet_tcp_socket_get_port:
- *  @socket: GTcpSocket to get the port number of.
+ *  gnet_tcp_socket_get_local_inetaddr
+ *  @socket: a #GTcpSocket
  *
- *  Get the port number the socket is bound to.
+ *  Gets the local host's address from a #GTcpSocket.
  *
- *  Returns: Port number of the socket.
+ *  Returns: a #GInetAddr.
+ *
+ **/
+GInetAddr*  
+gnet_tcp_socket_get_local_inetaddr (const GTcpSocket* socket)
+{
+  socklen_t socklen;
+  struct sockaddr_storage sa;
+  GInetAddr* ia;
+
+  g_return_val_if_fail (socket, NULL);
+
+  socklen = sizeof(sa);
+  if (getsockname(socket->sockfd, &GNET_SOCKADDR_SA(sa), &socklen) != 0)
+    return NULL;
+
+  ia = g_new0(GInetAddr, 1);
+  ia->ref_count = 1;
+  ia->sa = sa;
+
+  return ia;
+}
+
+
+/**
+ *  gnet_tcp_socket_get_port
+ *  @socket: a #GTcpSocket
+ *
+ *  Gets the port a server #GTcpSocket is bound to.
+ *
+ *  Returns: the port number.
  *
  **/
 gint
-gnet_tcp_socket_get_port(const GTcpSocket* socket)
+gnet_tcp_socket_get_port (const GTcpSocket* socket)
 {
   g_return_val_if_fail (socket != NULL, 0);
 
-  return g_ntohs(GNET_SOCKADDR_IN(socket->sa).sin_port);
+  return g_ntohs(GNET_SOCKADDR_PORT(socket->sa));
 }
 
 
+
 /* **************************************** */
 
 /**
- *  gnet_tcp_socket_set_tos:
- *  @socket: GTcpSocket to set the type-of-service of
- *  @tos: Type of service (in tcp.h)
+ *  gnet_tcp_socket_set_tos
+ *  @socket: a #GTcpSocket
+ *  @tos: type of service
  *
- *  Set the type-of-service of the socket.  Usually, routers don't
- *  honor this.  Some systems don't support this function.  If the
- *  operating system does not support it, the function does nothing.
+ *  Sets the type-of-service (TOS) of the socket.  TOS theoretically
+ *  controls the connection's quality of service, but most routers
+ *  ignore it.  Some systems don't even support this function.  The
+ *  function does nothing if the operating system does not support it.
  *
  **/
 void
@@ -792,127 +855,116 @@ gnet_tcp_socket_set_tos (GTcpSocket* socket, GNetTOS tos)
 
 
 /**
- *  gnet_tcp_socket_server_new:
- *  @port: Port number for the socket (0 if you don't care).
+ *  gnet_tcp_socket_server_new
  *
- *  Create and open a new #GTcpSocket with the specified port number.
- *  Use this sort of socket when you are a server and you know what
- *  the port number should be (or pass 0 if you don't care what the
- *  port is).  SOCKS is used if SOCKS is enabled.
+ *  Creates a new #GTcpSocket bound to all interfaces and an arbitrary
+ *  port.  SOCKS is used if SOCKS is enabled.
  *
- *  Returns: a new #GTcpSocket, or NULL if there was a failure.
+ *  Returns: a new #GTcpSocket; NULL on error.
  *
  **/
 GTcpSocket* 
-gnet_tcp_socket_server_new (gint port)
+gnet_tcp_socket_server_new (void)
 {
-  GInetAddr iface;
-  struct sockaddr_in* sa_in;
-  
-  /* Use SOCKS if enabled */
-  if (gnet_socks_get_enabled())
-    return gnet_private_socks_tcp_socket_server_new (port);
-
-  /* Set up address and port (any address, any port) */
-  memset (&iface, 0, sizeof(iface));
-  sa_in = (struct sockaddr_in*) &iface.sa;
-  sa_in->sin_family = AF_INET;
-  sa_in->sin_addr.s_addr = g_htonl(INADDR_ANY);
-  sa_in->sin_port = g_htons(port);
-
-  return gnet_tcp_socket_server_new_interface (&iface);
+  return gnet_tcp_socket_server_new_full (NULL, 0);
 }
 
 
 /**
- *  gnet_tcp_socket_server_new_interface:
- *  @iface: Interface to bind to
+ *  gnet_tcp_socket_server_new_with_port
+ *  @port: port to bind to (0 for an arbitrary port)
  *
- *  Create and open a new #GTcpSocket bound to the specified
- *  interface.  Use this sort of socket when your are a server and
- *  have a specific address the server must be bound to.  If the port
- *  number of the interface address is 0, the OS will use the
- *  interface specified but choose the port itself.  If the interface
- *  address is NULL, the OS will choose both the interface and port.
- *  If the interface address was created by gnet_inetaddr_new_any(),
- *  the OS will use all interfaces.  If the port number is also set,
- *  it will use that port number.  SOCKS is used if SOCKS is enabled
- *  and the interface is NULL.
+ *  Creates a new #GTcpSocket bound to all interfaces and port @port.
+ *  If @port is 0, an arbitrary port will be used.  SOCKS is used if
+ *  SOCKS is enabled.
  *
- *  Returns: a new #GTcpSocket, or NULL if there was a failure.
+ *  Returns: a new #GTcpSocket; NULL on error.
  *
  **/
 GTcpSocket* 
-gnet_tcp_socket_server_new_interface (const GInetAddr* iface)
+gnet_tcp_socket_server_new_with_port (gint port)
 {
+  return gnet_tcp_socket_server_new_full (NULL, port);
+}
+
+
+/**
+ *  gnet_tcp_socket_server_new_full
+ *  @iface: Interface to bind to (NULL for all interfaces)
+ *  @port: Port to bind to (0 for an arbitrary port)
+ *
+ *  Creates and new #GTcpSocket bound to interface @iface and port
+ *  @port.  If @iface is NULL, the socket is bound to all interfaces.
+ *  If @port is 0, the socket is bound to an arbitrary port.  SOCKS is
+ *  used if SOCKS is enabled and the interface is NULL.
+ *
+ *  Returns: a new #GTcpSocket; NULL on error.
+ *
+ **/
+GTcpSocket* 
+gnet_tcp_socket_server_new_full (const GInetAddr* iface, gint port)
+{
+  int sockfd = 0;
+  struct sockaddr_storage sa;
   GTcpSocket* s;
-  struct sockaddr_in* sa_in;
   socklen_t socklen;
 
   /* Use SOCKS if enabled */
   if (!iface && gnet_socks_get_enabled())
-    return gnet_private_socks_tcp_socket_server_new (0);
+    return gnet_private_socks_tcp_socket_server_new (port);
 
-  /* Create socket */
-  s = g_new0(GTcpSocket, 1);
-  s->ref_count = 1;
-  s->sockfd = socket(AF_INET, SOCK_STREAM, 0);
-  if (s->sockfd < 0)
-    goto error;
-
-  /* Set up address and port for connection */
-  sa_in = (struct sockaddr_in*) &s->sa;
-  sa_in->sin_family = AF_INET;
-  if (iface)
-    {
-      sa_in->sin_addr.s_addr = GNET_SOCKADDR_IN(iface->sa).sin_addr.s_addr;
-      sa_in->sin_port = GNET_SOCKADDR_IN(iface->sa).sin_port;
-    }
-  else
-    {
-      sa_in->sin_addr.s_addr = g_htonl(INADDR_ANY);
-      sa_in->sin_port = 0;
-    }
+  /* Create sockfd and address */
+  sockfd = gnet_private_create_listen_socket (SOCK_STREAM, iface, port, &sa);
+  if (sockfd < 0)
+    return NULL;
 
   /* The socket is set to non-blocking mode later in the Windows
      version.*/
-#ifndef GNET_WIN32
+#ifndef GNET_WIN32		/* Unix */
   {
     gint flags;
     const int on = 1;
     /* Set REUSEADDR so we can reuse the port */
-    if (setsockopt(s->sockfd, SOL_SOCKET, SO_REUSEADDR, 
+    if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, 
 		   (void*) &on, sizeof(on)) != 0)
       g_warning("Can't set reuse on tcp socket\n");
 
     /* Get the flags (should all be 0?) */
-    flags = fcntl(s->sockfd, F_GETFL, 0);
+    flags = fcntl(sockfd, F_GETFL, 0);
     if (flags == -1)
       goto error;
     
     /* Make the socket non-blocking */
-    if (fcntl(s->sockfd, F_SETFL, flags | O_NONBLOCK) == -1)
+    if (fcntl(sockfd, F_SETFL, flags | O_NONBLOCK) == -1)
       goto error;
   }
 #endif
 
   /* Bind */
-  if (bind(s->sockfd, &s->sa, sizeof(s->sa)) != 0)
+  if (bind(sockfd, &GNET_SOCKADDR_SA(sa), GNET_SOCKADDR_LEN(sa)) != 0)
     goto error;
   
   /* Get the socket name */
-  socklen = sizeof(s->sa);
-  if (getsockname(s->sockfd, &s->sa, &socklen) != 0)
+  socklen = GNET_SOCKADDR_LEN(sa);
+  if (getsockname(sockfd, &GNET_SOCKADDR_SA(sa), &socklen) != 0)
     goto error;
   
   /* Listen */
-  if (listen(s->sockfd, 10) != 0)
+  if (listen(sockfd, 10) != 0)
     goto error;
   
+  /* Create TcpSocket */
+  s = g_new0(GTcpSocket, 1);
+  s->sockfd = sockfd;
+  s->sa = sa;
+  s->ref_count = 1;
+
   return s;
   
  error:
-  if (s)    		g_free(s);
+  if (sockfd)    		
+    GNET_CLOSE_SOCKET(sockfd);
+
   return NULL;
 }
 
@@ -923,24 +975,23 @@ gnet_tcp_socket_server_new_interface (const GInetAddr* iface)
 
 
 /**
- *  gnet_tcp_socket_server_accept:
- *  @socket: #GTcpSocket to accept connections from.
+ *  gnet_tcp_socket_server_accept
+ *  @socket: a #GTcpSocket
  *
- *  Accept a connection from the socket.  The socket must have been
- *  created using gnet_tcp_socket_server_new().  This function will
- *  block (use gnet_tcp_socket_server_accept_nonblock() if you don't
- *  want to block).  If the socket's #GIOChannel is readable, it DOES
- *  NOT mean that this function will not block.
+ *  Accepts a connection from a #GTcpSocket.  The socket must have
+ *  been created using gnet_tcp_socket_server_new() (or equivalent).
+ *  Even if the socket's #GIOChannel is readable, the function may
+ *  still block.
  *
- *  Returns: a new #GTcpSocket if there is another connect, or NULL if
- *  there's an error.
+ *  Returns: a new #GTcpSocket representing a new connection; NULL on
+ *  error.
  *
  **/
 GTcpSocket* 
 gnet_tcp_socket_server_accept (GTcpSocket* socket)
 {
   gint sockfd;
-  struct sockaddr sa;
+  struct sockaddr_storage sa;
   socklen_t n;
   fd_set fdset;
   GTcpSocket* s;
@@ -965,7 +1016,7 @@ gnet_tcp_socket_server_accept (GTcpSocket* socket)
 
   n = sizeof(s->sa);
   
-  if ((sockfd = accept(socket->sockfd, &sa, &n)) == -1)
+  if ((sockfd = accept(socket->sockfd, (struct sockaddr*) &sa, &n)) == -1)
     {
       if (errno == EWOULDBLOCK || 
 	  errno == ECONNABORTED ||
@@ -981,7 +1032,7 @@ gnet_tcp_socket_server_accept (GTcpSocket* socket)
   s = g_new0(GTcpSocket, 1);
   s->ref_count = 1;
   s->sockfd = sockfd;
-  memcpy(&s->sa, &sa, sizeof(s->sa));
+  s->sa = sa;
 
   return s;
 }
@@ -991,17 +1042,19 @@ gnet_tcp_socket_server_accept (GTcpSocket* socket)
 
 
 /**
- *  gnet_tcp_socket_server_accept_nonblock:
- *  @socket: GTcpSocket to accept connections from.
+ *  gnet_tcp_socket_server_accept_nonblock
+ *  @socket: a #GTcpSocket
  *
- *  Accept a connection from the socket without blocking.  The socket
- *  must have been created using gnet_tcp_socket_server_new().  This
- *  function is best used with the socket's #GIOChannel.  If the
- *  channel is readable, then you PROBABLY have a connection.  It is
- *  possible for the connection to close by the time you call this, so
- *  it may return NULL even if the channel was readable.
+ *  Accepts a connection from a #GTcpSocket without blocking.  The
+ *  socket must have been created using gnet_tcp_socket_server_new()
+ *  (or equivalent).
  *
- *  Returns a new GTcpSocket if there is another connect, or NULL
+ *  Note that if the socket's #GIOChannel is readable, then there is
+ *  PROBABLY a new connection.  It is possible for the connection
+ *  to close by the time this function is called, so it may return
+ *  NULL.
+ *
+ *  Returns: a new #GTcpSocket representing a new connection; NULL
  *  otherwise.
  *
  **/
@@ -1009,7 +1062,7 @@ GTcpSocket*
 gnet_tcp_socket_server_accept_nonblock (GTcpSocket* socket)
 {
   gint sockfd;
-  struct sockaddr sa;
+  struct sockaddr_storage sa;
   socklen_t n;
   fd_set fdset;
   GTcpSocket* s;
@@ -1034,7 +1087,7 @@ gnet_tcp_socket_server_accept_nonblock (GTcpSocket* socket)
     }
 
   n = sizeof(sa);
-  if ((sockfd = accept(socket->sockfd, &sa, &n)) == -1)
+  if ((sockfd = accept(socket->sockfd, (struct sockaddr*) &sa, &n)) == -1)
     {
       /* If we get an error, return.  We don't want to try again as we
          do in gnet_tcp_socket_server_accept() - it might cause a
@@ -1046,7 +1099,7 @@ gnet_tcp_socket_server_accept_nonblock (GTcpSocket* socket)
   s = g_new0(GTcpSocket, 1);
   s->ref_count = 1;
   s->sockfd = sockfd;
-  memcpy(&s->sa, &sa, sizeof(s->sa));
+  s->sa = sa;
 
   return s;
 }
@@ -1059,9 +1112,10 @@ GTcpSocket*
 gnet_tcp_socket_server_accept (GTcpSocket* socket)
 {
   gint sockfd;
-  struct sockaddr sa;
+  struct sockaddr_storage sa;
   fd_set fdset;
   GTcpSocket* s;
+  socklen_t n;
 
   g_return_val_if_fail (socket != NULL, NULL);
 
@@ -1072,24 +1126,21 @@ gnet_tcp_socket_server_accept (GTcpSocket* socket)
   FD_SET((unsigned)socket->sockfd, &fdset);
 
   if (select(socket->sockfd + 1, &fdset, NULL, NULL, NULL) == -1)
-    {
       return NULL;
-    }
 
   /* Don't force the socket into blocking mode */
 
-  sockfd = accept(socket->sockfd, &sa, NULL);
+  n = sizeof(sa);
+  sockfd = accept(socket->sockfd, (struct sockaddr*) &sa, &n);
   /* if it fails, looping isn't going to help */
 
   if (sockfd == INVALID_SOCKET)
-    {
       return NULL;
-    }
 
   s = g_new0(GTcpSocket, 1);
   s->ref_count = 1;
   s->sockfd = sockfd;
-  memcpy(&s->sa, &sa, sizeof(s->sa));
+  s->sa = sa;
 
   return s;
 }
@@ -1099,11 +1150,12 @@ GTcpSocket*
 gnet_tcp_socket_server_accept_nonblock (GTcpSocket* socket)
 {
   gint sockfd;
-  struct sockaddr sa;
+  struct sockaddr_storage sa;
 
   fd_set fdset;
   GTcpSocket* s;
   u_long arg;
+  socklen_t n;
 
   g_return_val_if_fail (socket != NULL, NULL);
 
@@ -1123,7 +1175,8 @@ gnet_tcp_socket_server_accept_nonblock (GTcpSocket* socket)
   if(ioctlsocket(socket->sockfd, FIONBIO, &arg))
     return NULL;
 
-  sockfd = accept(socket->sockfd, &sa, NULL);
+  n = sizeof(sa);
+  sockfd = accept(socket->sockfd, (struct sockaddr*) &sa, &n);
   /* if it fails, looping isn't going to help */
 
   if (sockfd == INVALID_SOCKET)
@@ -1134,7 +1187,7 @@ gnet_tcp_socket_server_accept_nonblock (GTcpSocket* socket)
   s = g_new0(GTcpSocket, 1);
   s->ref_count = 1;
   s->sockfd = sockfd;
-  memcpy(&s->sa, &sa, sizeof(s->sa));
+  s->sa = sa;
 
   return s;
 }
@@ -1150,12 +1203,14 @@ static gboolean tcp_socket_server_accept_async_cb (GIOChannel* iochannel,
 
 /**
  *  gnet_tcp_socket_server_accept_async:
- *  @socket: #GTcpSocket to accept connections from.
- *  @accept_func: Callback function.
- *  @user_data: User data passed when callback function is called.
+ *  @socket: a #GTcpSocket
+ *  @accept_func: callback function.
+ *  @user_data: data to pass to @func on callback
  *
- *  Accept a connection from the socket asynchronously.  The callback
- *  is called when a client has connection or the socket has an error.
+ *  Asynchronously accepts a connection from a #GTcpSocket.  The
+ *  callback is called when a new client has connected or an error
+ *  occurs.  The socket must have been created using
+ *  gnet_tcp_socket_server_new() (or equivalent).
  *
  **/
 void
@@ -1180,11 +1235,10 @@ gnet_tcp_socket_server_accept_async (GTcpSocket* socket,
   socket->accept_data = user_data;
 
   /* Add read watch */
-  iochannel = gnet_tcp_socket_get_iochannel (socket);
+  iochannel = gnet_tcp_socket_get_io_channel (socket);
   socket->accept_watch = g_io_add_watch(iochannel, 
 					G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL, 
 					tcp_socket_server_accept_async_cb, socket);
-  g_io_channel_unref (iochannel);
 }
 
 
@@ -1233,11 +1287,11 @@ tcp_socket_server_accept_async_cb (GIOChannel* iochannel, GIOCondition condition
 
 
 /**
- *  gnet_tcp_socket_server_accept_async_cancel:
- *  @socket: #GTcpSocket accepting connections asynchronously.
+ *  gnet_tcp_socket_server_accept_async_cancel
+ *  @socket: a #GTcpSocket
  *
- *  Stops accepting connections asynchronously from the socket.  This
- *  does not close the socket.
+ *  Stops asynchronously accepting connections for a #GTcpSocket.  The
+ *  socket is not closed.
  *
  **/
 void
